
#include "cgD3D.h"
#include "cgD3D_ab.h"

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

const char * cgProgramContainer::GetVertexDeclaration()
{
  return cgProgramContainer_GetVertexDeclaration(pc);
}
                
HRESULT cgProgramContainer::SetShaderConstantD3DXMATRIX(cgBindIter * iter, const D3DXMATRIX *m, int nArrayElements, int ArrayOffset)
{
  return cgProgramContainer_SetShaderConstantD3DXMATRIX(pc, iter, m, nArrayElements, ArrayOffset);
}

HRESULT cgProgramContainer::SetShaderConstant(cgBindIter * iter, void * data, int nArrayElements, int ArrayOffset)
{
  return cgProgramContainer_SetShaderConstant(pc, iter, data, nArrayElements, ArrayOffset);
}

int cgProgramContainer::GetTexturePosition(cgBindIter * BindIter)
{
  return cgProgramContainer_GetTexturePosition(pc, BindIter);
}

int cgProgramContainer::GetTexturePosition(const char * name)
{
  return cgProgramContainer_GetTexturePositionB(pc, name);
}

HRESULT cgProgramContainer::SetTexture(const char * name, LPDIRECT3DBASETEXTURE8 pTexture)
{
  return cgProgramContainer_SetTexture(pc, name, pTexture);
}

HRESULT cgProgramContainer::SetTexture(cgBindIter * BindIter, LPDIRECT3DBASETEXTURE8 pTexture)
{
  return cgProgramContainer_SetTextureB(pc, BindIter, pTexture);
}

HRESULT cgProgramContainer::SetTextureStageState(cgBindIter * BindIter, D3DTEXTURESTAGESTATETYPE Type,DWORD Value)
{
  return cgProgramContainer_SetTextureStageState(pc, BindIter, Type, Value);
}

HRESULT cgProgramContainer::SetTextureWrapMode(cgBindIter * BindIter, DWORD WrapCoords)
{
  return cgProgramContainer_SetTextureWrapMode(pc, BindIter, WrapCoords);
}


HRESULT cgProgramContainer::SetShaderActive()
{
  return cgProgramContainer_SetShaderActive(pc);
}


cgBindIter *cgProgramContainer::GetVertexBindByName(const char *vertex_attr_name)
{
  return cgProgramContainer_GetVertexBindByName(pc, vertex_attr_name);
}
cgBindIter *cgProgramContainer::GetTextureBindByName(const char *texture_name)
{
  return cgProgramContainer_GetTextureBindByName(pc, texture_name);
}
cgBindIter *cgProgramContainer::GetParameterBindByName(const char *parameter_name)
{
  return cgProgramContainer_GetParameterBindByName(pc, parameter_name);
}

void cgProgramContainer::FreeBindIter(cgBindIter *i)
{
  return cgProgramContainer_FreeBindIter(pc, i);
}

cgBindIter *cgProgramContainer::GetBindByFullName(const char *parameter_name)
{
  return cgProgramContainer_GetBindByFullName(pc, parameter_name);
}

const char *cgProgramContainer::GetProgramName()
{
  return cgProgramContainer_GetProgramName(pc);
}

const char *cgProgramContainer::GetProgramObjectCode()
{
  return cgProgramContainer_GetProgramObjectCode(pc);
}

cgProfileType cgProgramContainer::GetProgramProfile()
{
  return cgProgramContainer_GetProgramProfile(pc);
}

DWORD cgProgramContainer::GetShaderHandle()
{
  return cgProgramContainer_GetShaderHandle(pc);
}


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

const char *cgContextContainer::GetLastListing()
{
  return cgContextContainer_GetLastListing(cc);
}

cgProgramContainer * cgContextContainer::LoadCGProgramFromFile(
        const char * filename,           // filename of the Cg program
        const char * title,               // title of the program
        cgProfileType type,       // profile of the program
        cgVertexDefinition * va, // optionally, set streams or allow different vetex definitions
        DWORD * outIndex,
        const char * entry)
{
  void *pc = cgContextContainer_LoadCGProgramFromFile(cc,
                filename,           // filename of the Cg program
                title,               // title of the program
                type,       // profile of the program
                va, // optionally, set streams or allow different vetex definitions
                outIndex,
                entry );

  if (pc)
        return new cgProgramContainer( pc );
  else
        return NULL;
}

cgProgramContainer *  cgContextContainer::LoadCGProgramFromMemory(
        const char * memory,          // memory location that contain the cg program
        const char * title,            // name of the program
        cgProfileType type,    // profile of the program
        cgVertexDefinition * va,   // optionally, set the streams for each vertex entry
        DWORD * outIndex,
        const char * entry)
{
  void *pc = cgContextContainer_LoadCGProgramFromMemory(cc,
                memory,          // memory location that contain the cg program
                title,            // name of the program
                type,    // profile of the program
                va,   // optionally, set the streams for each vertex entry
                outIndex,
                entry );

  if (pc)                          
        return new cgProgramContainer( pc );
  else
        return NULL;
}


cgProgramContainer * cgContextContainer::GetProgramIterator(DWORD index)
{
  return new cgProgramContainer( cgContextContainer_GetProgramIterator(cc, index) );
}


cgProgramIter *cgContextContainer::GetNextProgram(cgProgramIter *iter)
{
  return cgContextContainer_GetNextProgram( cc, iter );
}

cgProgramIter *cgContextContainer::ProgramByName(const char *name)
{
  return cgContextContainer_ProgramByName( cc, name );
}


DWORD cgContextContainer::GetShaderHandle(cgProfileType type)
{
  return cgContextContainer_GetShaderHandle( cc, type );
}

cgContextContainer::~cgContextContainer( )
{
  cgContextContainer_Destroy( (cgContextContainer*)cc );
}


///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

cgDirect3D::cgDirect3D()
{
  cd3d = cgDirect3D_Create();
}

cgDirect3D::~cgDirect3D()
{
  cgDirect3D_Destroy( (cgDirect3D*)cd3d );
}

void cgDirect3D::AddFilePath(const char * dirpath)
{
  cgDirect3D_AddFilePath( cd3d, dirpath );
}

//    void AddFilePath(std::string & path);


void cgDirect3D::NotePad(const char * title, const char * listing )
{
  cgDirect3D_NotePad( cd3d, title, listing );
}

cgContextContainer * cgDirect3D::CreateContextContainer(LPDIRECT3DDEVICE8 pd3dDevice, DWORD * outIndex)
{
  void *cc = cgDirect3D_CreateContextContainer( cd3d, pd3dDevice, outIndex );

  if (cc)
        return new cgContextContainer( cc );
  else
        return NULL;
}

